/* ------------------------------------------------------------------------
 * ImageImpl.cc
 *
 * This file is part of 3Dwm: The Three-Dimensional User Environment.
 *
 * 3Dwm: The Three-Dimensional User Environment:
 *	<http://www.3dwm.org>
 *
 * Chalmers Medialab
 * 	<http://www.medialab.chalmers.se>
 * 
 * ------------------------------------------------------------------------
 * File created 2000-09-18 by Steve Houston.
 *
 * Copyright (c) 2000, 2001 Niklas Elmqvist <elm@3dwm.org>.
 * Copyright (c) 2000 Steve Houston <steve@3dwm.org>.
 * ------------------------------------------------------------------------
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * ------------------------------------------------------------------------
 */

// -- System includes
#include <stdio.h>
#include <stdlib.h>

// -- 3Dwm includes
#include "Celsius/debug.hh"
#include "Polhem/ImageImpl.hh"

using namespace Nobel;


// -- Code Segment

/*
 *************************************
 * ImageImpl - handles image loading *
 *************************************
 */

ImageImpl::ImageImpl() 
    : _width(0), _height(0)
{
    DPRINT("ImageImpl::ImageImpl\n");
}

ImageImpl::~ImageImpl()
{

}

void ImageImpl::loadRaw(const BinaryData &raw_data, CORBA::Long width,
			CORBA::Long height, Nobel::PixelType pixelType)
{
    _binData = raw_data;
    _width = width;
    _height = height;
    _pixelType = pixelType;
}

/*
 * Free the memory used by the current image
 */
void ImageImpl::free()
{

}

/*
 ********************************************************
 * TextureImpl - extends ImageImpl to handle loading of *
 * texture images                                       *
 ********************************************************
 */

TextureImpl::TextureImpl()  
{
    DPRINT("TextureImpl::TextureImpl\n");
    _isReady = false;
    _isSplit = false;
}

TextureImpl::~TextureImpl()
{

}

CORBA::Long TextureImpl::init(Renderer_ptr r)
{

    // TODO: ensure _width, _height etc. are initialized
    if (!r->genTexture(_width, _height, _pixelType, _binData, 
		      _texName))
	// There was a problem initialising the texture.
	// Return without enabling texturing.
	return TEX_INIT_FAIL;
    else
	_isReady = 1;

    return 0;
}

void TextureImpl::apply(Renderer_ptr r)
{
    r->applyTexture(_texName);
}

void TextureImpl::free()
{
    ImageImpl::free();
    // TODO: somehow we need to get a message to the renderer to issue
    // a glDeleteTextures() on our texurename.
}

void TextureImpl::isReady(const CORBA::Boolean rdy)
{
    _isReady = rdy;
}

CORBA::Boolean TextureImpl::isReady()
{
    return _isReady;
}

Tag TextureImpl::getName(CORBA::Long index)
{
    return (CORBA::Long)_texNameArray[index];
}

/*
 ********************************************************
 * ProceduralTextureImpl                                *
 ********************************************************
 */

ProceduralTextureImpl::ProceduralTextureImpl()
{
    // empty
}

ProceduralTextureImpl::~ProceduralTextureImpl()
{
    // empty
}

// initTexture - called before the texture is used.
// Returns 0 if success,
// TEX_SPLIT if the texture had to be split up to work on current 
// hardware
// TEX_INIT_FAIL if unable to initialize texture.
//
CORBA::Long ProceduralTextureImpl::init(Renderer_ptr r)
{
    // Get the initial texture
    BinaryData_var tex = _texGen->getTextureData();
    if (r->genTexture(_width, _height, _pixelType, tex.in(), _texName) == 0)
	return TEX_INIT_FAIL;
    _isReady = 1;
    
    return 0;
}

void ProceduralTextureImpl::apply(Renderer_ptr r)
{ 
    BinaryData_var tex;
    Rectangle rect;
    
    // TODO: make sure texGen is initialized
    if (_texGen->getUpdate(tex.out(), rect))
	r->updateTexture(rect, _pixelType, tex.in(), _texName);
    
    // Now apply the texture to the renderer
    r->applyTexture(_texName);
}

void ProceduralTextureImpl::setTextureGenerator(TextureGenerator_ptr texGen)
{
    // Sanity check
    if (CORBA::is_nil(texGen))
	return;

    // Update values
    _texGen = TextureGenerator::_duplicate(texGen);
    _width = _texGen->getWidth();
    _height = _texGen->getHeight();
    _pixelType = _texGen->getPixelType();
}
